@@ -213,9 +213,10 @@ Fl_Widget_Type::Fl_Widget_Type() {
 
 Fl_Widget_Type::~Fl_Widget_Type() {
   if (o) {
-    o->hide();
-    if (o->parent()) ((Fl_Group*)o->parent())->remove(*o);
+    Fl_Window *win = o->window();
     delete o;
+    if (win)
+      win->redraw();
   }
   if (subclass_) free((void*)subclass_);
   if (tooltip_) free((void*)tooltip_);
@@ -2015,6 +2016,7 @@ int isdeclare(const char *c) {
 void Fl_Widget_Type::write_static() {
   const char* t = subclassname(this);
   if (!subclass() || (is_class() && !strncmp(t, "Fl_", 3))) {
+    write_declare("#include <FL/Fl.H>");
     write_declare("#include <FL/%s.H>", t);
   }
   for (int n=0; n < NUM_EXTRA_CODE; n++) {
@@ -2067,7 +2069,7 @@ void Fl_Widget_Type::write_static() {
     write_c(", %s", ut);
     if (use_v) write_c(" v");
     write_c(") {\n");
-    write_c_indented(callback());
+    write_c_indented(callback(), 1, 0);
     if (*(d-1) != ';' && *(d-1) != '}') {
       const char *p = strrchr(callback(), '\n');
       if (p) p ++;
@@ -2199,7 +2201,9 @@ void Fl_Widget_Type::write_code1() {
 
   indentation++;
 
-  if (wused) write_c("%sw = o; if (w) {/* empty */}\n", indent());
+  // Avoid compiler warning for unused variable.
+  // Also avoid quality control warnings about incorrect allocation error handling.
+  if (wused) write_c("%sw = o; (void)w;\n", indent());
 
   write_widget_code();
 }
